@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,15 +41,13 @@
  *
  */
 
-#define WE_DONT_WANT_NO_SOCKADDR_ARG_UNION
-
 #include "common.h"
 
 #if SOCKSLIBRARY_DYNAMIC
 
 
 static const char rcsid[] =
-"$Id: interposition.c,v 1.74 2001/02/06 15:58:22 michaels Exp $";
+"$Id: interposition.c,v 1.79 2005/01/24 10:24:19 karls Exp $";
 
 #undef accept
 #undef bind
@@ -58,6 +56,9 @@ static const char rcsid[] =
 #undef gethostbyaddr
 #undef gethostbyname
 #undef gethostbyname2
+#undef getaddrinfo
+#undef getipnodebyname
+#undef freehostent
 #undef getpeername
 #undef getsockname
 #undef read
@@ -98,6 +99,13 @@ static struct libsymbol_t libsymbolv[] = {
 #if HAVE_GETHOSTBYNAME2
 	{	SYMBOL_GETHOSTBYNAME2,			LIBRARY_GETHOSTBYNAME2,	NULL,	NULL },
 #endif /* HAVE_GETHOSTBYNAME2 */
+#if HAVE_GETADDRINFO
+	{	SYMBOL_GETADDRINFO,				LIBRARY_GETADDRINFO,	NULL,	NULL },
+#endif /* HAVE_GETADDRINFO */
+#if HAVE_GETIPNODEBYNAME
+	{	SYMBOL_GETIPNODEBYNAME,			LIBRARY_GETIPNODEBYNAME,	NULL,	NULL },
+ 	{	SYMBOL_FREEHOSTENT,			LIBRARY_FREEHOSTENT,	NULL,	NULL },
+#endif /* HAVE_GETIPNODEBYNAME */
 #endif /* SOCKS_CLIENT */
 	{	SYMBOL_GETHOSTBYNAME,			LIBRARY_GETHOSTBYNAME,	NULL,	NULL },
 
@@ -462,7 +470,7 @@ sys_send(s, msg, len, flags)
 	SEND_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_SEND);
+	function = (SEND_FUNC_T)symbolfunction(SYMBOL_SEND);
 	rc = function(s, msg, len, flags);
 	SYSCALL_END(s);
 	return rc;
@@ -848,6 +856,83 @@ gethostbyname2(name, af)
 	return Rgethostbyname2(name, af);
 }
 
+#if HAVE_GETADDRINFO
+
+int
+sys_getaddrinfo(nodename, servname, hints, res)
+	const char *nodename;
+	const char *servname;
+	const struct addrinfo *hints;
+	struct addrinfo **res;
+{
+	typedef int (*GETADDRINFO_FUNC_T)(const char *, const char *, 
+					  const struct addrinfo *,
+					  struct addrinfo **);
+	GETADDRINFO_FUNC_T function;
+
+	function = (GETADDRINFO_FUNC_T)symbolfunction(SYMBOL_GETADDRINFO);
+	return function(nodename, servname, hints, res);
+}
+
+int
+getaddrinfo(nodename, servname, hints, res)
+	const char *nodename;
+	const char *servname;
+	const struct addrinfo *hints;
+	struct addrinfo **res;
+{
+	return Rgetaddrinfo(nodename, servname, hints, res);
+}
+
+#endif /* HAVE_GETADDRINFO */
+
+#if HAVE_GETIPNODEBYNAME
+
+struct hostent *
+sys_getipnodebyname(name, af, flags, error_num)
+	const char *name;
+	int af;
+	int flags;
+	int *error_num;
+{
+	typedef struct hostent *(*GETIPNODEBYNAME_FUNC_T)(const char *, int, int, int *);
+	GETIPNODEBYNAME_FUNC_T function;
+
+	function = (GETIPNODEBYNAME_FUNC_T)symbolfunction(SYMBOL_GETIPNODEBYNAME);
+	return function(name, af, flags, error_num);
+}
+
+struct hostent *
+getipnodebyname(name, af, flags, error_num)
+	const char *name;
+	int af;
+	int flags;
+	int *error_num;
+{
+	return Rgetipnodebyname(name, af, flags, error_num);
+}
+
+void
+sys_freehostent(ptr)
+        struct hostent *ptr;
+{
+        typedef struct hostent *(*FREEHOSTENT_FUNC_T)(struct hostent *);
+
+	FREEHOSTENT_FUNC_T function;
+
+	function = (FREEHOSTENT_FUNC_T)symbolfunction(SYMBOL_FREEHOSTENT);
+	function(ptr);
+}
+
+void 
+freehostent(ptr)
+        struct hostent *ptr;
+{
+        Rfreehostent(ptr);
+}
+
+#endif /* HAVE_GETIPNODEBYNAME */
+
 #endif /* SOCKS_CLIENT */
 
 #endif /* SOCKSLIBRARY_DYNAMIC */
