@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,7 @@
  *  Software Distribution Coordinator  or  sdc@inet.no
  *  Inferno Nettverk A/S
  *  Oslo Research Park
- *  Gaustadalllllen 21
+ *  Gaustadallen 21
  *  NO-0349 Oslo
  *  Norway
  *
@@ -48,7 +48,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: msproxy_clientprotocol.c,v 1.25 2001/02/06 15:58:57 michaels Exp $";
+"$Id: msproxy_clientprotocol.c,v 1.30 2003/07/01 13:21:30 michaels Exp $";
 
 static char executable[] = "TELNET.EXE";
 static struct sigaction oldsigio;
@@ -144,6 +144,7 @@ msproxy_negotiate(s, control, packet)
 	struct socks_t *packet;
 {
 	const char *function = "msproxy_negotiate()";
+	char string[MAXSOCKADDRSTRING];
 	struct msproxy_request_t req;
 	struct msproxy_response_t res;
 	int p;
@@ -241,7 +242,7 @@ msproxy_negotiate(s, control, packet)
 	slog(LOG_DEBUG, "%s: msproxy controladdress: %s",
 	function,
 	sockaddr2string((struct sockaddr *)&packet->state.msproxy.controladdr,
-	NULL, 0));
+	string, sizeof(string)));
 
 
 	slog(LOG_DEBUG, "%s: packet #2", function);
@@ -292,6 +293,7 @@ msproxy_connect(s, control, packet)
 	struct socks_t *packet;
 {
 	const char *function = "msproxy_connect()";
+	char string[MAXSOCKSHOSTSTRING];
 	struct msproxy_request_t req;
 	struct msproxy_response_t res;
 	struct sockaddr_in addr;
@@ -472,12 +474,12 @@ msproxy_connect(s, control, packet)
 
 	if (socks_connect(s, &packet->res.host) != 0) {
 		swarn("%s: failed to connect to %s",
-		function, sockshost2string(&packet->res.host, NULL, 0));
+		function, sockshost2string(&packet->res.host, string, sizeof(string)));
 		return -1;
 	}
 	else
 		slog(LOG_DEBUG, "%s: connected to %s",
-		function, sockshost2string(&packet->res.host, NULL, 0));
+		function, sockshost2string(&packet->res.host, string, sizeof(string)));
 
 	packet->res.host.atype					= SOCKS_ADDR_IPV4;
 	packet->res.host.port					= res.packet._5.clientport;
@@ -485,7 +487,7 @@ msproxy_connect(s, control, packet)
 
 	/* LINTED pointer casts may be troublesome */
 	slog(LOG_DEBUG, "%s: server will use as source address: %s",
-	function, sockshost2string(&packet->res.host, NULL, 0));
+	function, sockshost2string(&packet->res.host, string, sizeof(string)));
 
 
 	slog(LOG_DEBUG, "%s: packet #6", function);
@@ -515,6 +517,7 @@ msproxy_bind(s, control, packet)
 	struct socks_t *packet;
 {
 	const char *function = "msproxy_bind()";
+	char string[MAXSOCKSHOSTSTRING];
 	struct msproxy_request_t req;
 	struct msproxy_response_t res;
 	struct sockaddr_in addr;
@@ -559,8 +562,7 @@ msproxy_bind(s, control, packet)
 	packet->res.host.addr.ipv4.s_addr	= res.packet._3.boundaddr;
 
 	slog(LOG_DEBUG, "%s: server bound for us: %s",
-	function, sockshost2string(&packet->res.host, NULL, 0));
-
+	function, sockshost2string(&packet->res.host, string, sizeof(string)));
 
 	slog(LOG_DEBUG, "%s: packet #4", function);
 
@@ -678,6 +680,7 @@ sigio(sig)
 {
 	const char *function = "sigio()";
 	const int errno_s = errno;
+	char string[MAXSOCKSHOSTSTRING];
 	int i, max, dset;
 	struct socksfd_t *socksfd;
 	fd_set rset;
@@ -787,22 +790,21 @@ sigio(sig)
 						sockshost2sockaddr(&host, &socksfd->remote);
 
 						slog(LOG_DEBUG, "%s: server bound address %s",
-						function, sockshost2string(&host, NULL, 0));
+						function, sockshost2string(&host, string, sizeof(string)));
 
 						host.atype					= SOCKS_ADDR_IPV4;
 						host.port					= res.packet._5.clientport;
 						host.addr.ipv4.s_addr	= res.packet._5.clientaddr;
-						sockshost2sockaddr(&host, &socksfd->accepted);
+						sockshost2sockaddr(&host, &socksfd->forus.accepted);
 
 						slog(LOG_DEBUG, "%s: server accepted: %s",
-						function, sockshost2string(&host, NULL, 0));
+						function, sockshost2string(&host, string, sizeof(string)));
 
 						slog(LOG_DEBUG, "%s: server will connect to us from port %d",
 						function, ntohs(res.packet._5.serverport));
 
 						/* LINTED pointer casts may be troublesome */
-						((struct sockaddr_in *)&socksfd->reply)->sin_port
-						= res.packet._5.serverport;
+						TOIN(&socksfd->reply)->sin_port = res.packet._5.serverport;
 
 						slog(LOG_DEBUG, "%s: packet #6", function);
 
