@@ -1,4 +1,4 @@
-/* $Id: seteuid.c,v 1.1 1999/09/29 10:18:03 karls Exp $ */
+/* $Id: seteuid.c,v 1.2 2004/11/09 07:10:24 karls Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "autoconf.h"
@@ -6,6 +6,14 @@
 
 #include "common.h"
 
+#if !HAVE_SETEGID
+int
+setegid(gid_t egid)
+{
+	return setresgid(-1, egid, -1);
+}
+#endif /* !HAVE_SETEGID */
+
 #if !HAVE_SETEUID
 
 int
