@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd_socket.c,v 1.33 2001/12/11 12:26:50 michaels Exp $";
+"$Id: sockd_socket.c,v 1.36 2003/07/01 13:21:49 michaels Exp $";
 
 int
 sockd_bind(s, addr, retries)
@@ -68,7 +68,18 @@ sockd_bind(s, addr, retries)
 
 			socks_seteuid(&euid, sockscf.uid.privileged);
 			/* LINTED pointer casts may be troublesome */
-			p = bindresvport(s, TOIN(addr));
+			if ((p = bind(s, addr, sizeof(*addr))) == -1 && errno == EADDRINUSE) {
+#if HAVE_BINDRESVPORT
+				/*
+				 * There are some differences in whether bindresvport()
+				 * retries or not on different systems, and Linux
+				 * ignores the portnumber altogether, so we have to
+				 * do two calls.
+				 */
+				TOIN(addr)->sin_port = htons(0);
+				p = bindresvport(s, TOIN(addr));
+#endif /* HAVE_BINDRESVPORT */
+			}
 			socks_reseteuid(sockscf.uid.privileged, euid);
 		}
 		else if ((p = bind(s, addr, sizeof(*addr))) == 0) {
